#version 120
//Horizontal bilateral blur for volumetric fog + Forward rendered objects + Draw volumetric fog
#extension GL_EXT_gpu_shader4 : enable
#define SEA_LEVEL 70 //[0 10 20 30 40 50 60 70 80 90 100 110 120 130 150 170 190]	//The volumetric light uses an altitude-based fog density, this is where fog density is the highest, adjust this value according to your world.



flat varying vec3 zMults;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D colortex7;
uniform sampler2D colortex8; // specular
uniform sampler2D colortex9; // specular
uniform sampler2D colortex3;
uniform sampler2D colortex2;
// uniform sampler2D colortex4;
uniform sampler2D colortex1;
uniform sampler2D colortex0;
uniform sampler2D noisetex;

flat varying vec4 lightCol; //main light source color (rgb),used light source(1=sun,-1=moon)
flat varying vec3 WsunVec;

uniform vec3 sunVec;
uniform ivec2 eyeBrightnessSmooth;
uniform float frameTimeCounter;
uniform int frameCounter;
uniform float far;
uniform float near;
uniform int isEyeInWater;
uniform vec3 chunkOffset;   
uniform float nightVision;
uniform float rainStrength;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform vec2 texelSize;
uniform float eyeAltitude;
uniform float blindness;
uniform sampler2D specular;
uniform vec3 cameraPosition;
#include "lib/waterBump.glsl"
#include "lib/waterOptions.glsl"
#include "/lib/res_params.glsl"

#include "lib/sky_gradient.glsl"
#include "lib/volumetricClouds.glsl"
#include "lib/biome_specifics.glsl"

float ld(float depth) {
    return 1.0 / (zMults.y - depth * zMults.z);		// (-depth * (far - near)) = (2.0 * near)/ld - far - near
}
#define diagonal3(m) vec3((m)[0].x, (m)[1].y, m[2].z)
#define  projMAD(m, v) (diagonal3(m) * (v) + (m)[3].xyz)
vec3 toScreenSpace(vec3 p) {
	vec4 iProjDiag = vec4(gbufferProjectionInverse[0].x, gbufferProjectionInverse[1].y, gbufferProjectionInverse[2].zw);
    vec3 p3 = p * 2. - 1.;
    vec4 fragposition = iProjDiag * p3.xyzz + gbufferProjectionInverse[3];
    return fragposition.xyz / fragposition.w;
}
vec4 BilateralUpscale(sampler2D tex, sampler2D depth,vec2 coord,float frDepth){
  coord = coord;
  vec4 vl = vec4(0.0);
  float sum = 0.0;
  mat3x3 weights;
  const ivec2 scaling = ivec2(1.0/VL_RENDER_RESOLUTION);
  ivec2 posD = ivec2(coord*VL_RENDER_RESOLUTION)*scaling;
  ivec2 posVl = ivec2(coord*VL_RENDER_RESOLUTION);
  float dz = zMults.x;
  ivec2 pos = (ivec2(gl_FragCoord.xy) % 2 )*2;
	//pos = ivec2(1,-1);

  ivec2 tcDepth =  posD + ivec2(-2,-2) * scaling + pos * scaling;
  float dsample = ld(texelFetch2D(depth,tcDepth,0).r);
  float w = abs(dsample-frDepth) < dz ? 1.0 : 1e-5;
  vl += texelFetch2D(tex,posVl+ivec2(-2)+pos,0)*w;
  sum += w;

	tcDepth =  posD + ivec2(-2,0) * scaling + pos * scaling;
  dsample = ld(texelFetch2D(depth,tcDepth,0).r);
  w = abs(dsample-frDepth) < dz ? 1.0 : 1e-5;
  vl += texelFetch2D(tex,posVl+ivec2(-2,0)+pos,0)*w;
  sum += w;

	tcDepth =  posD + ivec2(0) + pos * scaling;
  dsample = ld(texelFetch2D(depth,tcDepth,0).r);
  w = abs(dsample-frDepth) < dz ? 1.0 : 1e-5;
  vl += texelFetch2D(tex,posVl+ivec2(0)+pos,0)*w;
  sum += w;

	tcDepth =  posD + ivec2(0,-2) * scaling + pos * scaling;
  dsample = ld(texelFetch2D(depth,tcDepth,0).r);
  w = abs(dsample-frDepth) < dz ? 1.0 : 1e-5;
  vl += texelFetch2D(tex,posVl+ivec2(0,-2)+pos,0)*w;
  sum += w;

  return vl/sum;
}
float luma(vec3 color) {
	return dot(color,vec3(0.21, 0.72, 0.07));
}
vec3 decode (vec2 encn){
    vec3 n = vec3(0.0);
    encn = encn * 2.0 - 1.0;
    n.xy = abs(encn);
    n.z = 1.0 - n.x - n.y;
    n.xy = n.z <= 0.0 ? (1.0 - n.yx) * sign(encn) : encn;
    return clamp(normalize(n.xyz),-1.0,1.0);
}
vec2 decodeVec2(float a){
    const vec2 constant1 = 65535. / vec2( 256., 65536.);
    const float constant2 = 256. / 255.;
    return fract( a * constant1 ) * constant2 ;
}
//faster and actually more precise than pow 2.2
vec3 toLinear(vec3 sRGB){
	return sRGB * (sRGB * (sRGB * 0.305306011 + 0.682171111) + 0.012522878);
}

#define fsign(a)  (clamp((a)*1e35,0.,1.)*2.-1.)
float triangularize(float dither)
{
    float center = dither*2.0-1.0;
    dither = center*inversesqrt(abs(center));
    return clamp(dither-fsign(center),0.0,1.0);
}
vec3 fp10Dither(vec3 color,float dither){
	const vec3 mantissaBits = vec3(6.,6.,5.);
	vec3 exponent = floor(log2(color));
	return color + dither*exp2(-mantissaBits)*exp2(exponent);
}
float R2_dither(){
	vec2 alpha = vec2(0.75487765, 0.56984026);
	return fract(alpha.x * gl_FragCoord.x + alpha.y * gl_FragCoord.y + 1.0/1.6180339887 * frameCounter) ;
}


void main() {
  vec2 texcoord = gl_FragCoord.xy*texelSize;
  /* DRAWBUFFERS:73 */
  //3x3 bilateral upscale from half resolution
  float z = texture2D(depthtex0,texcoord).x;
  float z2 = texture2D(depthtex1,texcoord).x;
  float frDepth = ld(z);
  vec4 vl = BilateralUpscale(colortex0,depthtex0,gl_FragCoord.xy,frDepth);



  vec4 transparencies = texture2D(colortex2,texcoord);
  vec4 trpData = texture2D(colortex7,texcoord);
  bool iswater = trpData.a > 0.99;
  float iswater2 = trpData.a;

	vec4 data = texture2D(colortex1,texcoord);
	vec4 dataUnpacked1 = vec4(decodeVec2(data.z),decodeVec2(data.w));
	bool hand = (abs(dataUnpacked1.w-0.75) < 0.01);

  vec2 refractedCoord = texcoord;

	vec4 darain = texture2D(colortex9,texcoord);
 
  if (iswater){
    vec3 fragpos = toScreenSpace(vec3(texcoord-vec2(0.0)*texelSize*0.5,z));
  	vec3 np3 = mat3(gbufferModelViewInverse) * fragpos + gbufferModelViewInverse[3].xyz + cameraPosition;
    float norm = getWaterHeightmap(np3.xz, iswater2)-0.5;
    float displ = norm/(length(fragpos)/far)/2000. * (1.0 + isEyeInWater*2.0);
    refractedCoord += displ*RENDER_SCALE;

    if (texture2D(colortex7,refractedCoord).a < 0.99) refractedCoord = texcoord;
  }

 vec3 color = texture2D(colortex3,refractedCoord).rgb;

 if (frDepth > 2.5/far || transparencies.a < 0.99 || !hand) color = color * (1.0-transparencies.a) + transparencies.rgb*10.; // Discount fix for transparencies through hand


	float dirtAmount = Dirt_Amount;
	vec3 waterEpsilon = vec3(Water_Absorb_R, Water_Absorb_G, Water_Absorb_B);
	vec3 dirtEpsilon = vec3(Dirt_Absorb_R, Dirt_Absorb_G, Dirt_Absorb_B);
	vec3 totEpsilon = dirtEpsilon*dirtAmount + waterEpsilon;
	vec3 scatterCoef = dirtAmount * vec3(Dirt_Scatter_R, Dirt_Scatter_G, Dirt_Scatter_B);

  color *= vl.a ;

  vec3 fragpos = toScreenSpace(vec3(texcoord-vec2(0.0)*texelSize*0.5,z));

	float lightleakfix = clamp(eyeBrightnessSmooth.y/240.0,0.0,1.0);


  /// blindness effect
  // if (isEyeInWater == 0){
    // color.rgb *=   mix(vec3(1), clamp(exp((-length(fragpos)+1) * vec3(0.6,1,0.6)),0.0,1.0) ,blindness);
  // }
  // vl.a *=  mix(1, exp(-length(fragpos)/15) ,blindness);

  // underwater fog 
  if (isEyeInWater == 1){
    vec3 fragpos1 = toScreenSpace(vec3(texcoord,z2));
    vec3 fragpos = toScreenSpace(vec3(texcoord,z));
  	vec3 np3 = mat3(gbufferModelViewInverse) * fragpos1 + gbufferModelViewInverse[3].xyz + cameraPosition;
    float abovewater = clamp(63 - np3.y,0.0,1.0);
    if (abovewater >= 1.0 )  fragpos = toScreenSpace(vec3(texcoord,z));
    // if(trpData.a > 1.01 ) fragpos = toScreenSpace(vec3(texcoord,z2));
     color.rgb *= exp(-length(fragpos)/2*totEpsilon);
    vl.a *= (dot(exp(-length(fragpos)/1.2*totEpsilon),vec3(0.2,0.7,0.1)))*0.5+0.5;
    
  }

  /// lava fog.
  if (isEyeInWater == 2){
    color.rgb *= exp(-length(fragpos)*vec3(0.2,0.7,4.0)*4.);
    color.rgb += vec3(4.0,0.5,0.1)*0.25;
    vl.a = 0.0;
  }

  //cave fog
  if (isEyeInWater == 0){
    color.rgb = mix(color.rgb, (vec3(0.025,0.05,0.1)*0.5) ,  clamp( pow( length(fragpos)/far, 10), 0, 1) * (1.0-lightleakfix)  );
    
  // 	vec3 np3 = mat3(gbufferModelViewInverse) * fragpos + gbufferModelViewInverse[3].xyz + cameraPosition;

	// 	vec3 testc = skyFromTex(np3,colortex4)/150. + toLinear(texture2D(colortex1,texcoord).rgb)/10.*4.0*ffstep(0.985,-dot(lightCol.a*WsunVec,np3));
  //  if(z < 1.0) color.rgb = mix(color.rgb, vec3(0) ,  clamp( pow( length(fragpos)/far, 1), 0, 1)  );
  }

  // if(darain.a < 1.0/255.0) vl.a += mix(255.0,1.0, pow(darain.a,0.00001) ) ;
  color += vl.rgb ;

  float borderfog = clamp( pow( length(fragpos)/far , 5), 0, 1);

  
  vec3 np3 = mat3(gbufferModelViewInverse) * fragpos + gbufferModelViewInverse[3].xyz + cameraPosition;
	vec3 bordercolor = skyFromTex(np3,colortex4)/150.;
	vec3 finalborderfog = clamp(fp10Dither(bordercolor*8./3.,triangularize(R2_dither())),0.0,65000.);
  
//  if(z < 1.0) color.rgb = bordercolor;

  // color.rgb *= bordercolor*borderfog;
 
  gl_FragData[0].r = vl.a;
  gl_FragData[1].rgb = clamp(color,6.11*1e-5,65000.0);
  gl_FragData[1].rgb *= mix(vec3(1), vec3(0.5,2,0.5),  nightVision);
  
  // if(transparencies.a > 0 && !iswater) gl_FragData[1] = vec4(color,1) * vec4(vec3(luma(color))*transparencies.rgb,  transparencies.a)    ;
  // if(transparencies.a > 0 && !iswater) gl_FragData[1] = transparencies *  vec4(clamp(color,6.11*1e-5,65000.0),1.) * (1-transparencies.a)     ;




    	// float z = texture2D(depthtex0,texcoord).x;
    // vec3 fragpos = toScreenSpace(vec3(texcoord-vec2(0.0)*texelSize*0.5,z));

	// #include "lib/sky_gradient.glsl"


  // vec3 np3 = mat3(gbufferModelViewInverse) * fragpos + gbufferModelViewInverse[3].xyz + cameraPosition;

	// float borderfog = clamp( pow( length(fragpos)/far, 0.5), 0, 1);
	// vec3 borderfogcolor = skyFromTex(np3,colortex4)/150. + toLinear(texture2D(colortex1,texcoord).rgb)/10.*4.0*ffstep(0.985,-dot(lightCol.a*WsunVec,np3));
	// vec3 finalborderfog = clamp(fp10Dither(borderfogcolor*8./3.,triangularize(R2_dither())),0.0,65000.);
	// gl_FragData[1].rgb = mix(	gl_FragData[0].rgb, finalborderfog, borderfog);



// if (z >=1.0) {}else{
// 	gl_FragData[1].rgb = istranslucent.rgb;
// }
  // gl_FragData[1].a += vl.a;
  
}
